"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Mocker = void 0;
const cuid_1 = __importDefault(require("cuid"));
const PUPPET = __importStar(require("wechaty-puppet"));
const config_js_1 = require("../config.js");
const mod_js_1 = require("./user/mod.js");
const generator_js_1 = require("./generator.js");
class Mocker {
    id;
    cacheContactPayload;
    cacheRoomPayload;
    cacheMessagePayload;
    cacheFileBoxPayload;
    mockerifiedContactMock;
    mockerifiedMessageMock;
    mockerifiedRoomMock;
    get ContactMock() { return this.mockerifiedContactMock; }
    get MessageMock() { return this.mockerifiedMessageMock; }
    get RoomMock() { return this.mockerifiedRoomMock; }
    environmentList;
    cleanupFns;
    _puppet;
    set puppet(puppet) {
        if (this._puppet) {
            throw new Error('puppet has already been set before. can not be set twice.');
        }
        this._puppet = puppet;
    }
    get puppet() {
        if (!this._puppet) {
            throw new Error('puppet has not been set yet, cannot be used.');
        }
        return this._puppet;
    }
    constructor() {
        config_js_1.log.verbose('Mocker', 'constructor()');
        this.id = (0, cuid_1.default)();
        this.environmentList = [];
        this.cleanupFns = [];
        this.cacheContactPayload = new Map();
        this.cacheMessagePayload = new Map();
        this.cacheRoomPayload = new Map();
        this.cacheFileBoxPayload = new Map();
        this.mockerifiedContactMock = (0, mod_js_1.mockerifyContactMock)(this);
        this.mockerifiedMessageMock = (0, mod_js_1.mockerifyMessageMock)(this);
        this.mockerifiedRoomMock = (0, mod_js_1.mockerifyRoomMock)(this);
    }
    toString() {
        return `Mocker<${this.id}>`;
    }
    use(...behaviorList) {
        config_js_1.log.verbose('Mocker', 'use(%s)', behaviorList.length);
        this.environmentList.push(...behaviorList);
    }
    start() {
        config_js_1.log.verbose('Mocker', 'start()');
        this.environmentList.forEach(behavior => {
            config_js_1.log.verbose('Mocker', 'start() enabling behavior %s', behavior.name);
            const stop = behavior(this);
            this.cleanupFns.push(stop);
        });
    }
    stop() {
        config_js_1.log.verbose('Mocker', 'stop()');
        this.cleanupFns.forEach(setImmediate);
        this.cleanupFns.length = 0;
    }
    randomContact() {
        config_js_1.log.verbose('Mocker', 'randomContact()');
        const contactIdList = [...this.cacheContactPayload.keys()];
        if (contactIdList.length <= 0) {
            return;
        }
        const index = Math.floor(contactIdList.length * Math.random());
        const id = contactIdList[index];
        const payload = this.cacheContactPayload.get(id);
        if (!payload) {
            throw new Error('no payload');
        }
        return this.ContactMock.load(id);
    }
    randomRoom() {
        config_js_1.log.verbose('Mocker', 'randomRoom()');
        const roomIdList = [...this.cacheRoomPayload.keys()];
        if (roomIdList.length <= 0) {
            return;
        }
        const index = Math.floor(roomIdList.length * Math.random());
        const id = roomIdList[index];
        const payload = this.cacheRoomPayload.get(id);
        if (!payload) {
            throw new Error('no payload');
        }
        return this.RoomMock.load(id);
    }
    randomConversation() {
        config_js_1.log.verbose('Mocker', 'randomConversation()');
        const contactIdList = [...this.cacheContactPayload.keys()];
        const roomIdList = [...this.cacheRoomPayload.keys()];
        const total = contactIdList.length + roomIdList.length;
        if (total <= 0) {
            throw new Error('no conversation found: 0 contact & 0 room!');
        }
        const pickContact = contactIdList.length / total;
        let conversation;
        if (Math.random() < pickContact) {
            conversation = this.randomContact();
        }
        else { // const pickRoom = roomIdList.length / total
            conversation = this.randomRoom();
        }
        if (!conversation) {
            throw new Error('no conversation');
        }
        return conversation;
    }
    /**
     *
     * Events
     *
     */
    scan(qrcode, status = PUPPET.types.ScanStatus.Waiting) {
        config_js_1.log.verbose('Mocker', 'scan(%s, %s)', qrcode, status);
        this.puppet.emit('scan', { qrcode, status });
    }
    login(user) {
        config_js_1.log.verbose('Mocker', 'login(%s)', user);
        this.puppet.login(user.id);
    }
    /**
     *
     * Creators for MockContacts / MockRooms
     *
     */
    /**
     * create an contact by specifying the payload
     */
    createContact(payload) {
        config_js_1.log.verbose('Mocker', 'createContact(%s)', payload ? JSON.stringify(payload) : '');
        const defaultPayload = (0, generator_js_1.generateContactPayload)();
        const normalizedPayload = {
            ...defaultPayload,
            ...payload,
        };
        return this.ContactMock.create(normalizedPayload);
    }
    /**
     * create `num` contacts randomly
     */
    createContacts(num) {
        config_js_1.log.verbose('Mocker', 'createContacts(%s)', num);
        const contactList = [];
        while (num--) {
            const contact = this.createContact();
            contactList.push(contact);
        }
        return contactList;
    }
    /**
     * create a room by specifying the payload
     */
    createRoom(payload) {
        config_js_1.log.verbose('Mocker', 'createRoom(%s)', payload ? JSON.stringify(payload) : '');
        const defaultPayload = (0, generator_js_1.generateRoomPayload)(...this.cacheContactPayload.keys());
        const normalizedPayload = {
            ...defaultPayload,
            ...payload,
        };
        return this.RoomMock.create(normalizedPayload);
    }
    /**
     * create `num` rooms randomly
     */
    createRooms(num) {
        config_js_1.log.verbose('Mocker', 'createRooms(%s)', num);
        const roomList = [];
        while (num--) {
            const room = this.createRoom();
            roomList.push(room);
        }
        return roomList;
    }
    contactPayload(id, payload) {
        config_js_1.log.silly('Mocker', 'contactPayload(%s%s)', id, payload ? ',' + JSON.stringify(payload) : '');
        if (payload) {
            this.cacheContactPayload.set(id, payload);
            return;
        }
        payload = this.cacheContactPayload.get(id);
        if (!payload) {
            throw new Error('no payload found for id ' + id);
        }
        return payload;
    }
    roomPayload(id, payload) {
        config_js_1.log.silly('Mocker', 'roomPayload(%s%s)', id, payload ? ',' + JSON.stringify(payload) : '');
        if (payload) {
            this.cacheRoomPayload.set(id, payload);
            return;
        }
        payload = this.cacheRoomPayload.get(id);
        if (!payload) {
            throw new Error('no payload found for id ' + id);
        }
        return payload;
    }
    messagePayload(id, payload) {
        config_js_1.log.silly('Mocker', 'messagePayload(%s%s)', id, payload ? ',' + JSON.stringify(payload) : '');
        if (payload) {
            this.cacheMessagePayload.set(id, payload);
            const msg = this.MessageMock.load(payload.id);
            msg.room()?.emit('message', msg);
            msg.talker().emit('message', msg);
            msg.listener()?.emit('message', msg);
            return;
        }
        payload = this.cacheMessagePayload.get(id);
        if (!payload) {
            throw new Error('no payload found for id ' + id);
        }
        return payload;
    }
}
exports.Mocker = Mocker;
//# sourceMappingURL=mocker.js.map