import * as PUPPET from 'wechaty-puppet';
import type { PuppetMock } from '../puppet-mock.js';
import { ContactMock, RoomMock, MessageMock } from './user/mod.js';
import type { EnvironmentMock } from './environment.js';
import type { FileBoxJsonObject } from 'file-box/dist/esm/src/file-box.type.js';
declare class Mocker {
    id: string;
    cacheContactPayload: Map<string, PUPPET.payloads.Contact>;
    cacheRoomPayload: Map<string, PUPPET.payloads.Room>;
    cacheMessagePayload: Map<string, PUPPET.payloads.Message>;
    cacheFileBoxPayload: Map<string, FileBoxJsonObject>;
    protected mockerifiedContactMock?: typeof ContactMock;
    protected mockerifiedMessageMock?: typeof MessageMock;
    protected mockerifiedRoomMock?: typeof RoomMock;
    get ContactMock(): typeof ContactMock;
    get MessageMock(): typeof MessageMock;
    get RoomMock(): typeof RoomMock;
    protected environmentList: EnvironmentMock[];
    protected cleanupFns: (() => void)[];
    protected _puppet?: PuppetMock;
    set puppet(puppet: PuppetMock);
    get puppet(): PuppetMock;
    constructor();
    toString(): string;
    use(...behaviorList: EnvironmentMock[]): void;
    start(): void;
    stop(): void;
    randomContact(): undefined | ContactMock;
    randomRoom(): undefined | RoomMock;
    randomConversation(): ContactMock | RoomMock;
    /**
     *
     * Events
     *
     */
    scan(qrcode: string, status?: PUPPET.types.ScanStatus): void;
    login(user: ContactMock): void;
    /**
     *
     * Creators for MockContacts / MockRooms
     *
     */
    /**
     * create an contact by specifying the payload
     */
    createContact(payload?: Partial<PUPPET.payloads.Contact>): ContactMock;
    /**
     * create `num` contacts randomly
     */
    createContacts(num: number): ContactMock[];
    /**
     * create a room by specifying the payload
     */
    createRoom(payload?: Partial<PUPPET.payloads.Room>): RoomMock;
    /**
     * create `num` rooms randomly
     */
    createRooms(num: number): RoomMock[];
    /**
     *
     * Setters & Getters for Payloads
     *
     */
    contactPayload(id: string, payload: PUPPET.payloads.Contact): void;
    contactPayload(id: string): PUPPET.payloads.Contact;
    roomPayload(id: string, payload: PUPPET.payloads.Room): void;
    roomPayload(id: string): PUPPET.payloads.Room;
    messagePayload(id: string, payload: PUPPET.payloads.Message): void;
    messagePayload(id: string): PUPPET.payloads.Message;
}
export { Mocker };
//# sourceMappingURL=mocker.d.ts.map