// import { Attachment } from './types'
import * as PUPPET from 'wechaty-puppet';
import { log } from '../../config.js';
const POOL = Symbol('pool');
// const ATTACHMENT = Symbol('attachment')
class MessageMock {
    payload;
    static get mocker() { throw new Error('This class can not be used directory. See: https://github.com/wechaty/wechaty/issues/2027'); }
    get mocker() { throw new Error('This class can not be used directory. See: https://github.com/wechaty/wechaty/issues/2027'); }
    static [POOL];
    // protected static [ATTACHMENT]: Map<string, Attachment>
    static get pool() {
        if (!this[POOL]) {
            log.verbose('MockMessage', 'get pool() init pool');
            this[POOL] = new Map();
        }
        if (this === MessageMock) {
            throw new Error('The global MockMessage class can not be used directly!'
                + 'See: https://github.com/wechaty/wechaty/issues/1217');
        }
        return this[POOL];
    }
    // protected static get attachmentPool () {
    //   if (!this[ATTACHMENT]) {
    //     log.verbose('Mock Message', 'get attachment pool () init pool')
    //     this[ATTACHMENT] = new Map()
    //   }
    //   if (this === MessageMock) {
    //     throw new Error(
    //       'The global MockMessage class can not be used directly!'
    //       + 'See: https://github.com/wechaty/wechaty/issues/1217',
    //     )
    //   }
    //   return this[ATTACHMENT]
    // }
    /**
     * @ignore
     * About the Generic: https://stackoverflow.com/q/43003970/1123955
     *
     * @static
     * @param {string} id
     * @returns {MessageMock}
     */
    static load(id) {
        const existingMessage = this.pool.get(id);
        if (existingMessage) {
            return existingMessage;
        }
        throw new Error(`MockMessage.load(): ${id} not exist.`);
    }
    // public static setAttachment<T extends typeof MessageMock> (
    //   this: T,
    //   id: string,
    //   attachment: Attachment
    // ): void {
    //   log.verbose('MockMessage', 'static set attachment(%s) to (%s)', JSON.stringify(attachment), id)
    //   this.attachmentPool.set(id, attachment)
    // }
    // public static loadAttachment<T extends typeof MessageMock> (
    //   this : T,
    //   id   : string,
    // ): Attachment | undefined {
    //   return this.attachmentPool.get(id)
    // }
    static create(payload) {
        log.verbose('MockMessage', 'static create(%s)', JSON.stringify(payload));
        if (this.pool.get(payload.id)) {
            throw new Error('MockMessage id ' + payload.id + ' has already created before. Use `load(' + payload.id + ')` to get it back.');
        }
        // when we call `load()`, `this` should already be extend-ed a child class.
        // so we force `this as any` at here to make the call.
        const newMessage = new this(payload);
        this.pool.set(newMessage.id, newMessage);
        this.mocker.messagePayload(payload.id, payload);
        return newMessage;
    }
    get id() { return this.payload.id; }
    constructor(payload) {
        this.payload = payload;
        log.silly('MockMessage', 'constructor(%s)', JSON.stringify(payload));
    }
    talker() {
        log.verbose('MockMessage', 'talker()');
        if (!this.payload.talkerId) {
            throw new Error('no talkerId');
        }
        const contact = this.mocker.ContactMock.load(this.payload.talkerId);
        return contact;
    }
    room() {
        log.verbose('MockMessage', 'room()');
        if (!this.payload.roomId) {
            return;
        }
        const room = this.mocker.RoomMock.load(this.payload.roomId);
        return room;
    }
    listener() {
        log.verbose('MockMessage', 'listener()');
        if (!this.payload.listenerId) {
            return undefined;
        }
        const contact = this.mocker.ContactMock.load(this.payload.listenerId);
        return contact;
    }
    text() {
        log.verbose('MockMessage', 'text()');
        return this.payload.text;
    }
    type() {
        log.silly('MockMessage', 'text()');
        return this.payload.type;
    }
    async toContact() {
        log.verbose('MockMessage', 'toContact()');
        if (this.type() !== PUPPET.types.Message.Contact) {
            throw new Error('message not a ShareCard');
        }
        const contactId = await this.mocker.puppet.messageContact(this.id);
        if (!contactId) {
            throw new Error(`can not get Contact id by message: ${contactId}`);
        }
        const contact = await this.mocker.ContactMock.load(contactId);
        return contact;
    }
    // async toUrlLink (): Promise<UrlLink> {
    //   log.verbose('MockMessage', 'toUrlLink()')
    //   if (this.type() !== MessageType.Url) {
    //     throw new Error('message not a Url Link')
    //   }
    //   const urlLink = await this.mocker.puppet.messageUrl(this.id)
    //   return new UrlLink(urlLink)
    // }
    // async toMiniprogram (): Promise<MiniProgram> {
    //   log.verbose('MockMessage', 'toMiniProgram()')
    //   if (this.type() !== MessageType.MiniProgram) {
    //     throw new Error('message not a MiniProgram')
    //   }
    //   const miniprogram = await this.mocker.puppet.messageMiniProgram(this.id)
    //   return new MiniProgram(miniprogram)
    // }
    async toFileBox() {
        log.verbose('MockMessage', 'toFileBox()');
        if (this.type() === PUPPET.types.Message.Text) {
            throw new Error('message is a Text');
        }
        return this.mocker.puppet.messageFile(this.id);
    }
}
function mockerifyMessageMock(mocker) {
    class MockerifiedMessageMock extends MessageMock {
        static get mocker() { return mocker; }
        get mocker() { return mocker; }
    }
    return MockerifiedMessageMock;
}
export { MessageMock, mockerifyMessageMock, };
//# sourceMappingURL=message-mock.js.map